package lab3;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.List;

public class Main {

    public static void main(String[] args) {
        List<Person> persons = new ArrayList();
        try (BufferedReader in = new BufferedReader(new FileReader("persons.txt"))) {
            String line = in.readLine();
            while (line != null) {
                String[] values = line.split(",");
                for(int i=0; i<values.length; i++) {
                    values[i] = values[i].trim();
                }
                // Alternative implementation using streams, map, lambda expressions
                // Arrays.stream(values).map(String::trim).toArray(i -> values);
                Person person = new Person(values[0],Integer.parseInt(values[1]),values[2]);
                System.out.println("Person " + person + " read.");
                persons.add(person);
                line = in.readLine();
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        System.out.println();
        
        try (ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream("persons.data"))) {
            // We have to store the number of objects in the file first because the ObjectInputStream does not provide
            // any means to detect the end of the file.
            out.writeObject(persons.size());
            for(Person person : persons) {
                out.writeObject(person);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        
        try (ObjectInputStream in = new ObjectInputStream(new FileInputStream("persons.data"))) {
            int size = (int) in.readObject();
            for(int i=0; i<size; i++) {
                Person person = (Person) in.readObject();
                System.out.println("Person " + person + " read again.");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        System.out.println();
        
        try (ObjectRandomAccessFile file = new ObjectRandomAccessFile("persons.rdata","rw")) {
            int offset = 0;
            for(Person person : persons) {
                int size = file.writeObject(person);
                System.out.println(person.getName() + " written at " + offset + " size = " + size);
                offset += size;
            }
            System.out.println();
            int[] readOrder = {3,2,4,1,5,0};
            for(int i=0; i<readOrder.length; i++) {
                file.seekObject(readOrder[i]);
                Person person = (Person) file.readObject();
                System.out.println("Person " + readOrder[i] + " is " + person + ".");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
